#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Plug-in to open GIMP's vectors in 'Inkscape'. Version 0.4 for Linux and Windows.

Tested with GIMP-2.8.16 and Python-2.7.11 on Linux.
Tested with GIMP-2.8.16 and Python-2.7.10 on Windows 7.

License:
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
The GNU Public License is available at
http://www.gnu.org/copyleft/gpl.html
'''

#import shlex
import sys, os, gettext
import gtk
import subprocess as sub


### global #####################################################################

### plug-in filename (thanks Ofnuts !!!)
whoiam = os.path.abspath(sys.argv[0])

# if not launch within GIMP
def mssgBox(mess):
    flag = gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT
    msgBox = gtk.MessageDialog(None, flag, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, mess)
    msgBox.run()
    msgBox.destroy()

try:
    from gimpfu import *
except ImportError:
    print("Note: GIMP is needed, '%s' is a plug-in for it.\n"%whoiam)
    # launch from GUI? use mssgBox(text)
    mssgBox("Note: GIMP is needed, '%s' is a plug-in for it.\n"%whoiam)
    sys.exit(1)

# for locale characters in windows file system
def sys_file(f_name):
    sys_encoded = sys.getfilesystemencoding()
    if os.name == 'nt': coded = f_name.encode(sys_encoded)
    else: coded = f_name
    return(coded)

# initialize internationalisation in a user-locale
locale_user_dir = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( "Gimp_to_Inkscape", locale_user_dir, unicode=True )


### main function ##############################################################

def open_path_in_inkscape(image, layer, ink, outputFolder, sel, schel, rem):
    ### utility functions
    def render_svg_layer(img, sgv_f):
        """
        Rendering an SVG file as a GIMP 'img' top layer
        """
        resol_x = img.resolution[0]
        # image = pdb.file_svg_load(filename, raw_filename, resolution, width, height, paths)
        fileImage = pdb.file_svg_load(sgv_f, sgv_f, resol_x, 0, 0, 2)
        # , run_mode=RUN_INTERACTIVE; 'paths = 2' is for merge paths
        pdb.gimp_edit_copy(fileImage.layers[0])
        # pasted as a top layer in the img
        pasted = pdb.gimp_edit_paste(img.layers[0], True)
        pdb.gimp_floating_sel_to_layer(pasted)
        pdb.gimp_image_delete(fileImage)

    pdb.gimp_image_undo_group_start(image)
    # in GIMP - armoring (if no vectors 'layer' it won't start)
    if os.access(sys_file(outputFolder), os.W_OK):
        # construct the filename of vector exchange file in 'utf-8'
        xchange_utf = outputFolder + os.sep + layer.name + ".svg"
        gimp.progress_init(_("Saving to '") + xchange_utf)
    else:
        gimp.message(_("ERROR: folder '%s' isn't writable, plug-in has terminated.")\
            %outputFolder)
        return
    if os.name == 'nt': tail = "inkscape.exe"
    else: tail = "inkscape"
    if not ink.endswith(tail) : 
        gimp.message(_("ERROR: executable '%s' pose problem, plug-in has terminated.")\
            %ink)
        return

    # export which path
    if sel == 0:
        pdb.gimp_vectors_export_to_file(image, xchange_utf, None)
    elif sel == 1:
        pdb.gimp_vectors_export_to_file(image, xchange_utf, image.active_vectors)

    # plus inkscape with file, 'ink' should be ASCII. Save XCF file also possible?
    xchange_sys = sys_file(xchange_utf) # for Python, Win file system not 'utf-8'
    p = sub.Popen([ink, xchange_sys], stdin=sub.PIPE, stdout=sub.PIPE, stderr=sub.STDOUT)
    child_output, child_error = p.communicate()
    gimp.progress_init(_("Returning from inkscape..."))

    # after inkscape, GIMP import vectors or render?
    if schel == 1 : # vectors
        gimp.vectors_import_from_file(image, xchange_utf, TRUE, TRUE)
    elif schel == 2 :   # render
        render_svg_layer(image, xchange_utf)
        #svg_layer = pdb.gimp_file_load_layer(image, xchange_utf, run_mode=RUN_INTERACTIVE)
        #image.insert_layer(svg_layer, 0, -1)    #problem: wrong parameter type?
    elif schel  == 3 :   # both
        gimp.vectors_import_from_file(image, xchange_utf, TRUE, TRUE)
        render_svg_layer(image, xchange_utf)

    # cleanup vector file?
    if rem:
        os.remove(xchange_sys)

    pdb.gimp_displays_flush()
    pdb.gimp_image_undo_group_end(image)
    gimp.progress_init(_("Plug-in end!"))


### GIMP registring & UI #######################################################

register(
    "open-path-in-inkscape",
    _("\nThis plugin opens: active or all GIMP vectors in Inkscape\n")\
    +_("------------------------ Instructions: -------------------------\n")\
    +_("Before in GIMP, select the active path & its name (may use special characters).\n")\
    +_("After start, this name+'.svg' will be the save file for vector exchange.\n")\
    +_("In Inkscape: save changes and close. This return to GIMP and end the plug-in.\n")\
    +_("----------------------------------------------------------------- \n")\
    +_("Plug-in file-name:\n")+whoiam,
    "Open active or all vector layers to Inkscape and return to GIMP - V 0.4 beta",
    "MareroQ and rob_brz1",
    "GPL",
    "2016",
    _("Vectors to Inkscape &..."),
    "RGB*,GRAY*",
    [
        (PF_IMAGE, "image", "Input image", None),
        #(PF_DRAWABLE, "drawable", "Drawable", None),
        (PF_VECTORS, "filename", "Start of vector exchange file", None),
        # SET ME<<----- executable to program inkscape, probably for
        # Win "C:\Program Files\Inkscape\inkscape.exe", Linux "/usr/bin/inkscape"
        (PF_FILENAME, "ink", _("Inkscape executable file: "), "/usr/bin/inkscape"),
        # SET ME<<----- path to temporary or not (auxiliary) file
        (PF_DIRNAME, "outputFolder", _("Output directory to save:"), "/home/robert/Bureau/"),
        (PF_OPTION, "sel", _("Export:"), 0, (_("All paths"),_("Active path"))),
        (PF_OPTION, "schel", _("Return to GIMP with:"), 0, (_("Nothing"),_("Vectors"),\
            _("Render"),_("Vectors+Render"))),
        (PF_BOOL, "rem_1", _("Delete file? "), True)
    ],
    [],
    open_path_in_inkscape,
    menu="<Vectors>/",
    domain=("Gimp_to_Inkscape", locale_user_dir)
    )

main()
